@@ -17,6 +17,9 @@ class Enums:
     PENDING = 'Pendientes'
     AUTHORIZED = 'Autorizada'
     LIST_AUTHORIZE_ROLE = [SENT, RECEIVED, PENDING, AUTHORIZED]
+    COMMUNITY = 'Comunidad'
+    STANDARD = 'Estándar'
+    LIST_TYPE_INVITE_ROLE = [COMMUNITY, STANDARD]
 
 
 class Constants:
