@@ -1,19 +1,47 @@
-from django.http import Http404
-from common_structure_microservices.exception import GenericMicroserviceError
+# from common_structure_microservices.exception import GenericMicroserviceError
 from common_structure_microservices.messages import Messages
 from common_structure_microservices.responses import response
+from django.core.exceptions import ValidationError
+from django.db.models.query import QuerySet
+from django.http import Http404
+from django.shortcuts import get_object_or_404 as _get_object_or_404
 from rest_framework import generics, status
 from rest_framework.response import Response
 from rest_framework.settings import api_settings
 from rest_framework.viewsets import ViewSetMixin
 
-#from polaris.common.audit_task import AuditTask
-#from polaris.common.utilities import Enums
+
+# from polaris.common.audit_task import AuditTask
+# from polaris.common.utilities import Enums
+
+def get_object_or_404(queryset, *filter_args, **filter_kwargs):
+    """
+    Same as Django's standard shortcut, but make sure to also raise 404
+    if the filter_kwargs don't match the required types.
+    """
+    try:
+        return _get_object_or_404(queryset, *filter_args, **filter_kwargs)
+    except (TypeError, ValueError, ValidationError):
+        raise Http404
 
 
 class GenericViewSet(ViewSetMixin, generics.GenericAPIView):
 
     def get_queryset(self):
+        """
+        Get the list of items for this view.
+        This must be an iterable, and may be a queryset.
+        Defaults to using `self.queryset`.
+
+        This method should always be used rather than accessing `self.queryset`
+        directly, as `self.queryset` gets evaluated only once, and those results
+        are cached for all subsequent requests.
+
+        You may want to override this if you need to provide different
+        querysets depending on the incoming request.
+
+        (Eg. return a list of items that is specific to the user)
+        """
         assert self.queryset is not None, (
                 "'%s' should either include a `queryset` attribute, "
                 "or override the `get_queryset()` method."
@@ -21,13 +49,22 @@ class GenericViewSet(ViewSetMixin, generics.GenericAPIView):
         )
 
         queryset = self.queryset
-        queryset = queryset.all()
-
+        if isinstance(queryset, QuerySet):
+            # Ensure queryset is re-evaluated on each request.
+            queryset = queryset.all()
         return queryset
 
     def get_object(self):
+        """
+        Returns the object the view is displaying.
+
+        You may want to override this if you need to provide non-standard
+        queryset lookups.  Eg if objects are referenced using multiple
+        keyword arguments in the url conf.
+        """
         queryset = self.filter_queryset(self.get_queryset())
 
+        # Perform the lookup filtering.
         lookup_url_kwarg = self.lookup_url_kwarg or self.lookup_field
 
         assert lookup_url_kwarg in self.kwargs, (
@@ -38,20 +75,20 @@ class GenericViewSet(ViewSetMixin, generics.GenericAPIView):
         )
 
         filter_kwargs = {self.lookup_field: self.kwargs[lookup_url_kwarg]}
+        obj = get_object_or_404(queryset, **filter_kwargs)
 
-        obj = queryset.filter(**filter_kwargs)
-
+        # May raise a permission denied
         self.check_object_permissions(self.request, obj)
 
         return obj
 
-    #def do_audit(self, request, action):
-        #if self.is_audit():
-            #obj = AuditTask()
-            #obj.audit(request=request, action=action)
+    # def do_audit(self, request, action):
+    # if self.is_audit():
+    # obj = AuditTask()
+    # obj.audit(request=request, action=action)
 
-    #def is_audit(self):
-        #return True
+    # def is_audit(self):
+    # return True
 
 
 class CreateModelMixin:
@@ -64,7 +101,7 @@ class CreateModelMixin:
         serializer.is_valid(raise_exception=True)
         self.perform_create(serializer)
         headers = self.get_success_headers(serializer.data)
-        #self.do_audit(request, Enums.CREATE)
+        # self.do_audit(request, Enums.CREATE)
         return Response(response(data=serializer.data, message=Messages.CREATED_SUCCESSFULLY),
                         status=status.HTTP_201_CREATED, headers=headers)
 
@@ -123,7 +160,7 @@ class UpdateModelMixin:
             # forcibly invalidate the prefetch cache on the instance.
             instance._prefetched_objects_cache = {}
 
-        #self.do_audit(request, Enums.UPDATE)
+        # self.do_audit(request, Enums.UPDATE)
         return Response(response(data=serializer.data, message=Messages.UPDATED_SUCCESSFULLY))
 
     def perform_update(self, serializer):
@@ -142,7 +179,7 @@ class DestroyModelMixin:
     def destroy(self, request, *args, **kwargs):
         instance = self.get_object()
         self.perform_destroy(instance)
-        #self.do_audit(request, Enums.DELETE)
+        # self.do_audit(request, Enums.DELETE)
         return Response(response(message=Messages.DELETED_SUCCESSFULLY), status=status.HTTP_200_OK)
 
     def perform_destroy(self, instance):
